/*================================================================
  File:     2_EnablingQueryStore.sql

  Summary:  Intro to Query Store

  SQL Server Versions: SQL Server 2016
------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com

  (c) 2016, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

/*
	For use with SQL Server 2016
	Enable Query Store
	
	Expand AdventureWorks2016 database first,
	don't see anything with QueryStore

	Enable through the UI
*/

/*
	this is the default if you script it out
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = ON;
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE
	);
GO



/*
	this is what it's really setting
*/
USE [master]
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = ON
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE, 
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30), 
	DATA_FLUSH_INTERVAL_SECONDS = 900,  
	INTERVAL_LENGTH_MINUTES = 60, 
	MAX_STORAGE_SIZE_MB = 100, 
	QUERY_CAPTURE_MODE = ALL, 
	SIZE_BASED_CLEANUP_MODE = AUTO, 
	MAX_PLANS_PER_QUERY = 200)
GO


/*
	We'll change these for our demo*
	*Values may not be appropriate for your production environment
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE (
	OPERATION_MODE = READ_WRITE,
		/*	
			READ_WRITE = data collection; READ_ONLY = no data collection 
		*/
	CLEANUP_POLICY = (STALE_QUERY_THRESHOLD_DAYS = 30),  
		/*	
			how long to retain data in the query store, default is 30 (used to be 367) 
		*/
	DATA_FLUSH_INTERVAL_SECONDS = 60,
		/*	
			how frequently QS data is written to disk, default is 900 (15 minutes) 
			how much QS data are you willing to lose in case of server crash
		*/
	INTERVAL_LENGTH_MINUTES = 1, 
		/*	
			interval across which QS data is aggregated, default is 60 minutes
			use a smaller value if you need finer granularity or less time to detect and mitigate issues
			this will affect the size of the data (lower value = more data) 
			From BOL: Note that arbitrary values are not allowed - 
			you must use one of the following: 1, 5, 10, 15, 30, 60, and 1440 minutes) 
		*/
	MAX_STORAGE_SIZE_MB = 1024,
		/*	
			size of the QS (when the max is hit, status changes to READ_ONLY) 
			default is 100MB...what this needs to be depends on your workload 
			AND how much information you want to keep
		*/
	QUERY_CAPTURE_MODE = ALL,
		/* 
			are ALL queries captured, or only "relevant" ones? 
			ALL is default
			AUTO is infrequent queries and those with insignificant compile and execution duration are ignored
			the thresholds (execution count, compile and runtime duration) are internally determined
		*/
	SIZE_BASED_CLEANUP_MODE = AUTO,
		/* 
			Do you want QS to go into clean up mode as usage gets close to MAX_STORAGE_SIZE_MB? 
			default is ON
			it is recommended to activate this so that QS always run in read-write mode and is collecting data
		*/
	MAX_PLANS_PER_QUERY = 200);
		/*
			max number of plans to store for a query
		*/
GO


/*
	start up the DEMO2 scripts
	check current status
*/
USE [AdventureWorks2016];
GO
SELECT actual_state_desc, readonly_reason, desired_state_desc, current_storage_size_mb, 
    max_storage_size_mb, flush_interval_seconds, interval_length_minutes, 
    stale_query_threshold_days, size_based_cleanup_mode_desc, 
    query_capture_mode_desc, max_plans_per_query
FROM sys.database_query_store_options;
GO


/*
	What do we see in QS?
*/
USE [AdventureWorks2016];
GO

SELECT *
FROM sys.query_store_query_text;
GO

SELECT *
FROM sys.query_store_query;
GO

SELECT 
	qst.query_text_id, qsq.query_id, qsq.object_id, 
	qsq.count_compiles, qst.query_sql_text, qst.statement_sql_handle, qsq.query_hash
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id;
GO


SELECT *
FROM sys.query_store_plan;
GO

/*
	Queries executed 
*/
SELECT
	qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, qsp.last_execution_time,
	LocalLastExecutionTime = DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time),
	qst.query_sql_text, 
	qsp.query_plan, ConvertedPlan = TRY_CONVERT(XML, qsp.query_plan)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
WHERE qsp.last_execution_time > DATEADD(HOUR, -8, GETUTCDATE());
GO

/*
	Queries executed in the last 8 hours
	with multiple plans
*/
USE [AdventureWorks2016E];
GO
SELECT
	qsq.query_id, 
	COUNT(qsp.plan_id) AS PlanCount,
	--qsq.object_id, 
	LocalLastExecutionTime = MAX(DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time)),
	MAX(qst.query_sql_text)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
WHERE qsp.last_execution_time > DATEADD(HOUR, -8, GETUTCDATE())
GROUP BY qsq.query_id, qsq.object_id
HAVING COUNT(qsp.plan_id) > 1;
GO


/*
	Textual matching still in effect
*/
USE [AdventureWorks2016];
GO

SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [Color];
GO

SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY [color];
GO

SELECT [d].[SalesOrderID], [d].[SalesOrderDetailID], [d].[OrderQty], [p].[ProductID], 
[d].[UnitPrice], [p].[Name], [p].[Color]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Production].[Product] [p] ON [d].[ProductID] = [p].[ProductID]
ORDER BY 
[Color];
GO


/*
	these queries have the same query_hash
*/
SELECT
	qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, 
	LocalLastExecutionTime = DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time),
	qst.query_sql_text, 
	ConvertedPlan = TRY_CONVERT(XML, qsp.query_plan)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
JOIN (
	SELECT query_hash AS [query_hash], COUNT(query_id) AS [Count]
	FROM sys.query_store_query 
	GROUP BY query_hash
	HAVING COUNT(*) > 1
	) m on qsq.query_hash = m.query_hash
WHERE qst.query_sql_text like '%color%';
GO


/*
	turn it off...
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] 
	SET QUERY_STORE = OFF;
GO

/*
	everything is still in QS views
*/
USE [AdventureWorks2016];
GO

SELECT 
	qst.query_text_id, qsq.query_id, qsq.is_internal_query, qsq.object_id, 
	qsq.count_compiles, qst.query_sql_text, qst.statement_sql_handle, qsq.query_hash
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id;


/*
	statement to clear what's in the query store
*/
ALTER DATABASE [AdventureWorks2016] 
	SET QUERY_STORE CLEAR;


/*
	Enable QS
	Run some queries
	2_Run AW Queries2.ps
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2016] SET QUERY_STORE = ON;
GO

/*
	run a few ad-hoc queries manually
*/
USE [AdventureWorks2016];
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 44444;
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 55555;
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 66666;
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 43659;
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 53573;
GO

SELECT *
FROM Sales.SalesOrderDetail
WHERE SalesOrderID = 67485;
GO

/*
	flush what's in memory to disk
*/
USE [AdventureWorks2016];
GO
EXEC sp_query_store_flush_db;
GO

/*
	what's it's in QS
*/
SELECT
	qsq.query_id, qsp.plan_id, qsq.object_id, 
	qsq.count_compiles, 
	LocalLastExecutionTime = DATEADD(MINUTE, -(DATEDIFF(MINUTE, GETDATE(), GETUTCDATE())), qsp.last_execution_time),
	qst.query_sql_text, 
	ConvertedPlan = TRY_CONVERT(XML, qsp.query_plan)
FROM sys.query_store_query_text qst
JOIN sys.query_store_query qsq ON qst.query_text_id = qsq.query_text_id
JOIN sys.query_store_plan qsp ON qsq.query_id = qsp.query_id
ORDER BY LocalLastExecutionTime DESC;
GO


/*
	enable remote admin connections if you're not local
*/
sp_configure 'remote admin connections', 1

reconfigure
GO

/*
	connect as ADMIN
*/
USE [AdventureWorks2016E];
GO


/*
	You can see the data in these tables.
	THIS IS NOT SUPPORTED
	USE WITH CAUTION
*/
SELECT *
FROM [sys].[plan_persist_query_text];
GO

SELECT *
FROM [sys].[plan_persist_context_settings];
GO

SELECT *
FROM [sys].[plan_persist_plan];
GO

SELECT *
FROM [sys].[plan_persist_query];
GO

SELECT *
FROM [sys].[plan_persist_runtime_stats];
GO

SELECT *
FROM [sys].[plan_persist_runtime_stats_interval];
GO




